@@ -2,93 +2,46 @@ package Rstats::Container;
 use Object::Simple -base;
 
 use Rstats::Func;
-use Rstats::List;
+use Rstats::Container::List;
 use Carp 'croak';
-use Rstats::Vector;
 
-sub is_perl_array_class {
-  my $self  = shift;
-  
-  my $is;
-  eval { $is = $self->isa('Rstats::Array') };
-  
-  return $is;
-}
-
-sub is_perl_list_class {
-  my $self  = shift;
-  
-  my $is;
-  eval { $is = $self->isa('Rstats::List') };
-  
-  return $is;
-}
-
-sub decompose_elements {
-  my $self = shift;
-  
-  if ($self->is_perl_array_class) {
-    return $self->elements->decompose;
-  }
-  else {
-    croak "Can't call decompose_elements methods from list";
-  }
-}
+has 'elements' => sub { [] };
 
 my %types_h = map { $_ => 1 } qw/character complex numeric double integer logical/;
 
 sub _copy_attrs_to {
-  my ($self, $x2, $opt) = @_;
-  
-  $opt ||= {};
-  my $new_indexes = $opt->{new_indexes};
-  my $exclude = $opt->{exclude} || [];
-  my %exclude_h = map { $_ => 1 } @$exclude;
-  
-  # dim
-  $x2->{dim} = [@{$self->{dim}}] if !$exclude_h{dim} && exists $self->{dim};
+  my ($self, $x2, $new_indexs) = @_;
   
   # class
-  $x2->{class} =  [@{$self->{class}}] if !$exclude_h{class} && exists $self->{class};
+  $x2->{class} =  $self->{class} if exists $self->{class};
 
   # levels
-  $x2->{levels} = $self->{levels} if !$exclude_h{levels} && exists $self->{levels};
-  
-  # type
-  $x2->{type} = $self->{type} if !$exclude_h{type} && exists $self->{type};
+  $x2->{levels} = $self->{levels} if exists $self->{levels};
   
   # names
-  if (!$exclude_h{names} && exists $self->{names}) {
+  if (exists $self->{names}) {
     my $names = [];
-    my $index = $self->is_data_frame ? $new_indexes->[1] : $new_indexes->[0];
+    my $index = $self->is_data_frame ? $new_indexs->[1] : $new_indexs->[0];
     if (defined $index) {
       for my $i (@{$index->values}) {
         push @$names, $self->{names}[$i - 1];
       }
     }
-    else {
-      $names = [@{$self->{names}}];
-    }
     $x2->{names} = $names;
   }
   
   # dimnames
-  if (!$exclude_h{dimnames} && exists $self->{dimnames}) {
+  if (exists $self->{dimnames}) {
     my $new_dimnames = [];
     my $dimnames = $self->{dimnames};
     my $length = @$dimnames;
     for (my $i = 0; $i < $length; $i++) {
       my $dimname = $dimnames->[$i];
       if (defined $dimname && @$dimname) {
-        my $index = $new_indexes->[$i];
+        my $index = $new_indexs->[$i];
         my $new_dimname = [];
-        if (defined $index) {
-          for my $k (@{$index->values}) {
-            push @$new_dimname, $dimname->[$k - 1];
-          }
-        }
-        else {
-          $new_dimname = [@$dimname];
+        for my $k (@{$index->values}) {
+          push @$new_dimname, $dimname->[$k - 1];
         }
         push @$new_dimnames, $new_dimname;
       }
@@ -180,11 +133,11 @@ sub str {
       push @str, "[$dim_str]";
     }
     
-    # Vector
+    # Elements
     my @element_str;
     my $max_count = $length > 10 ? 10 : $length;
     my $is_character = $self->is_character;
-    my $elements = $self->decompose_elements;
+    my $elements = $self->elements;
     for (my $i = 0; $i < $max_count; $i++) {
       push @element_str, $self->_element_to_string($elements->[$i], $is_character);
     }
@@ -219,8 +172,9 @@ sub levels {
 sub clone {
   my ($self, %opt) = @_;
   
-  my $clone = Rstats::Func::c($opt{elements} || $self->decompose_elements);
-  $self->_copy_attrs_to($clone);
+  my $clone = {%$self};
+  $clone = bless $clone, ref $self;
+  $clone->{elements} = $opt{elements} if $opt{elements};
   
   return $clone;
 }
@@ -243,11 +197,11 @@ sub _name_to_index {
   
   my $e1_name = $x1_index->element;
   my $found;
-  my $names = $self->names->decompose_elements;
+  my $names = $self->names->elements;
   my $index;
   for (my $i = 0; $i < @$names; $i++) {
     my $name = $names->[$i];
-    if (Rstats::VectorFunc::equal($e1_name, $name)) {
+    if (Rstats::ElementFunc::equal($e1_name, $name)) {
       $index = $i + 1;
       $found = 1;
       last;
@@ -275,13 +229,7 @@ sub length {
 sub length_value {
   my $self = shift;
   
-  my $length;
-  if ($self->is_perl_array_class) {
-    $length = $self->elements->length_value;
-  }
-  else {
-    $length = @{$self->list}
-  }
+  my $length = @{$self->elements};
   
   return $length;
 }
@@ -292,8 +240,8 @@ sub is_na {
   my $x1 = Rstats::Func::to_c($_a1);
   
   my @a2_elements = map {
-    ref $_ eq  'Rstats::Type::NA' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE()
-  } @{$x1->decompose_elements};
+    ref $_ eq  'Rstats::Type::NA' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE()
+  } @{$x1->elements};
   my $x2 = Rstats::Func::array(\@a2_elements);
   $x2->mode('logical');
   
@@ -301,23 +249,25 @@ sub is_na {
 }
 
 sub as_list {
-  my $self = shift;
+  my $container = shift;
   
-  if ($self->is_perl_list_class) {
-    return $self;
-  }
-  else {
-    my $list = Rstats::List->new;
-    $list->list([Rstats::Func::c($self->decompose_elements)]);
-    
-    return $list;
-  }
+  return $container if $container->is_list;
+
+  my $list = Rstats::Container::List->new;
+  $list->elements($container->elements);
+  
+  return $list;
 }
 
 sub is_list {
-  my $self = shift;
+  my $container = shift;
+  
+  my $is_list;
+  eval {
+    $is_list = $container->isa('Rstats::Container::List');
+  };
 
-  return $self->is_perl_list_class ? Rstats::Func::TRUE() : Rstats::Func::FALSE();
+  return $is_list ? Rstats::Func::TRUE() : Rstats::Func::FALSE();
 }
 
 sub class {
@@ -500,7 +450,7 @@ sub as_matrix {
     $col = 1;
   }
   
-  my $x2_elements = [@{$self->decompose_elements}];
+  my $x2_elements = [@{$self->elements}];
   
   return Rstats::Func::matrix($x2_elements, $row, $col);
 }
@@ -508,7 +458,7 @@ sub as_matrix {
 sub as_array {
   my $self = shift;
   
-  my $self_elements = [@{$self->decompose_elements}];
+  my $self_elements = [@{$self->elements}];
   my $self_dim_elements = [@{$self->dim_as_array->values}];
   
   return $self->array($self_elements, $self_dim_elements);
@@ -517,7 +467,7 @@ sub as_array {
 sub as_vector {
   my $self = shift;
   
-  my $self_elements = [@{$self->decompose_elements}];
+  my $self_elements = [@{$self->elements}];
   
   return Rstats::Func::c($self_elements);
 }
@@ -560,10 +510,9 @@ sub as_complex {
   }
 
   my $x2;
-  my $x_tmp_elements = $x_tmp->decompose_elements;
+  my $x_tmp_elements = $x_tmp->elements;
   my @a2_elements = map { $_->as('complex') } @$x_tmp_elements;
-  $x2 = Rstats::Func::c(\@a2_elements);
-  $x_tmp->_copy_attrs_to($x2);
+  $x2 = $x_tmp->clone(elements => \@a2_elements);
   $x2->{type} = 'complex';
 
   return $x2;
@@ -576,15 +525,14 @@ sub as_double {
   
   my $x2;
   if ($self->is_factor) {
-    my $x2_elements = [map { $_->as_double } @{$self->decompose_elements}];
-    $x2 = Rstats::Func::c($x2_elements);
+    my $x2_elements = map { $_->as_double } @{$self->elements};
+    $x2 = Rstats::Func::c($self->elements);
     $x2->{type} = 'double';
   }
   else {
-    my $self_elements = $self->decompose_elements;
+    my $self_elements = $self->elements;
     my @a2_elements = map { $_->as('double') } @$self_elements;
-    $x2 = Rstats::Func::c(\@a2_elements);
-    $self->_copy_attrs_to($x2);
+    $x2 = $self->clone(elements => \@a2_elements);
     $x2->{type} = 'double';
   }
 
@@ -596,14 +544,13 @@ sub as_integer {
   
   my $x2;
   if ($self->is_factor) {
-    $x2 = Rstats::Func::c($self->decompose_elements);
+    $x2 = Rstats::Func::c($self->elements);
     $x2->{type} = 'integer';
   }
   else {
-   my $self_elements = $self->decompose_elements;
+   my $self_elements = $self->elements;
     my @a2_elements = map { $_->as_integer  } @$self_elements;
-    $x2 = Rstats::Func::c(\@a2_elements);
-    $self->_copy_attrs_to($x2);
+    $x2 = $self->clone(elements => \@a2_elements);
     $x2->{type} = 'integer';
   }
 
@@ -615,14 +562,13 @@ sub as_logical {
   
   my $x2;
   if ($self->is_factor) {
-    $x2 = Rstats::Func::c($self->decompose_elements);
+    $x2 = Rstats::Func::c($self->elements);
     $x2 = $x2->as_logical;
   }
   else {
-    my $self_elements = $self->decompose_elements;
+    my $self_elements = $self->elements;
     my @a2_elements = map { $_->as_logical } @$self_elements;
-    $x2 = Rstats::Func::c(\@a2_elements);
-    $self->_copy_attrs_to($x2);
+    $x2 = $self->clone(elements => \@a2_elements);
     $x2->{type} = 'logical';
   }
 
@@ -638,14 +584,14 @@ sub as_character {
   if ($self->is_factor) {
     my $levels = {};
     my $x_levels = $self->levels;
-    my $x_levels_elements = $x_levels->decompose_elements;
+    my $x_levels_elements = $x_levels->elements;
     my $levels_length = $x_levels->length->value;
     for (my $i = 1; $i <= $levels_length; $i++) {
-      my $x_levels_element = $x_levels_elements->[$i - 1];
+      my $x_levels_element = $x_levels->elements->[$i - 1];
       $levels->{$i} = $x_levels_element->value;
     }
 
-    my $self_elements =  $self->decompose_elements;
+    my $self_elements =  $self->elements;
     my $x2_elements = [];
     for my $self_element (@$self_elements) {
       if ($self_element->is_na) {
@@ -658,14 +604,11 @@ sub as_character {
       }
     }
     $x2 = Rstats::Func::c($x2_elements);
-    $self->_copy_attrs_to($x2)
   }
   else {
-    my $self_elements = $self->decompose_elements;
+    my $self_elements = $self->elements;
     my @a2_elements = map { $_->as_character } @$self_elements;
-    $x2 = Rstats::Func::c(\@a2_elements);
-    $self->_copy_attrs_to($x2);
-    
+    $x2 = $self->clone(elements => \@a2_elements);
     $x2->{type} = 'character';
   }
 
@@ -676,12 +619,13 @@ sub values {
   my $self = shift;
   
   if (@_) {
-    $self->elements(Rstats::Func::c($_[0])->elements);
+    my @elements = map { Rstats::ElementFunc::element($_) } @{$_[0]};
+    $self->{elements} = \@elements;
   }
   else {
-    my $values = $self->elements->values;
-    
-    return $values;
+    my @values = map { defined $_ ? $_->value : undef } @{$self->elements};
+  
+    return \@values;
   }
 }
 
@@ -690,13 +634,13 @@ sub value {
   
   my $e1 = $self->element(@_);
   
-  return defined $e1 ? $e1->value : undef;
+  return defined $e1 ? $e1->value : Rstats::ElementFunc::NA();
 }
 
 sub is_vector {
   my $self = shift;
   
-  my $is = ref $self eq 'Rstats::Array' && $self->dim->length_value == 0 ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ref $self eq 'Rstats::Container::Array' && $self->dim->length_value == 0 ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -704,7 +648,7 @@ sub is_vector {
 sub is_matrix {
   my $self = shift;
 
-  my $is = ref $self eq 'Rstats::Array' && $self->dim->length_value == 2 ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ref $self eq 'Rstats::Container::Array' && $self->dim->length_value == 2 ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -713,7 +657,7 @@ sub is_numeric {
   my $self = shift;
   
   my $is = ($self->{type} || '') eq 'double' || ($self->{type} || '') eq 'integer'
-    ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+    ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -721,7 +665,7 @@ sub is_numeric {
 sub is_double {
   my $self = shift;
   
-  my $is = ($self->{type} || '') eq 'double' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ($self->{type} || '') eq 'double' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -729,7 +673,7 @@ sub is_double {
 sub is_integer {
   my $self = shift;
   
-  my $is = ($self->{type} || '') eq 'integer' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ($self->{type} || '') eq 'integer' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -737,7 +681,7 @@ sub is_integer {
 sub is_complex {
   my $self = shift;
   
-  my $is = ($self->{type} || '') eq 'complex' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ($self->{type} || '') eq 'complex' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -745,7 +689,7 @@ sub is_complex {
 sub is_character {
   my $self = shift;
   
-  my $is = ($self->{type} || '') eq 'character' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ($self->{type} || '') eq 'character' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -753,7 +697,7 @@ sub is_character {
 sub is_logical {
   my $self = shift;
   
-  my $is = ($self->{type} || '') eq 'logical' ? Rstats::VectorFunc::TRUE() : Rstats::VectorFunc::FALSE();
+  my $is = ($self->{type} || '') eq 'logical' ? Rstats::ElementFunc::TRUE() : Rstats::ElementFunc::FALSE();
   
   return Rstats::Func::c($is);
 }
@@ -761,13 +705,13 @@ sub is_logical {
 sub is_data_frame {
   my $self = shift;
   
-  return ref $self eq 'Rstats::DataFrame' ? Rstats::Func::TRUE() : Rstats::Func::FALSE();
+  return ref $self eq 'Rstats::Container::DataFrame' ? Rstats::Func::TRUE() : Rstats::Func::FALSE();
 }
 
 sub is_array {
   my $self = shift;
   
-  my $is = ref $self eq 'Rstats::Array' && exists $self->{dim};
+  my $is = ref $self eq 'Rstats::Container::Array' && exists $self->{dim};
   
   return $is;
 }
@@ -802,7 +746,7 @@ sub dimnames {
   
   if (@_) {
     my $dimnames_list = shift;
-    if (ref $dimnames_list eq 'Rstats::List') {
+    if (ref $dimnames_list eq 'Rstats::Container::List') {
       my $length = $dimnames_list->length_value;
       my $dimnames = [];
       for (my $i = 0; $i < $length; $i++) {
@@ -816,7 +760,7 @@ sub dimnames {
         }
       }
       $self->{dimnames} = $dimnames;
-      
+
       if ($self->is_data_frame) {
         $self->{names} = [@{$self->{dimnames}->[1]}];
       }
@@ -881,8 +825,4 @@ sub colnames {
   }
 }
 
-=head1 NAME
-
-Rstats::Container - Container base class
-
 1;
